#include <AIV/Matrices.mqh>

#property indicator_separate_window 
#property indicator_buffers 30
#property indicator_plots 30

enum EMode
{
  Change,
  Average
};

input int InpDepth = 500;
input int InpPeriod = 0;
input int InpPeriodSlow = 50;
input EMode InpMode = Average;
input string InpSymbols = "EURUSD,GBPUSD";

int iStart;
int iMaxBars;
int iLastBars;

double C0[], C1[], C2[], C3[], C4[], C5[], C6[], C7[], C8[], C9[];
double S0[], S1[], S2[], S3[], S4[], S5[], S6[], S7[], S8[], S9[];
double F0[], F1[], F2[], F3[], F4[], F5[], F6[], F7[], F8[], F9[];

SSets iSeries[];

CSets * iSets;
CHelpers * iHelpers;

void OnDeinit(const int reason)
{
  delete(iSets);
  delete(iHelpers);
}

void OnInit()
{
  iSets = new CSets();
  iHelpers = new CHelpers();
  
  iMaxBars = TerminalInfoInteger(TERMINAL_MAXBARS) - MathMax(InpPeriodSlow, InpPeriod) - 1;
  
  SetIndexBuffer(0, C0, INDICATOR_DATA); ArraySetAsSeries(C0, true);
  SetIndexBuffer(1, C1, INDICATOR_DATA); ArraySetAsSeries(C1, true);
  SetIndexBuffer(2, C2, INDICATOR_DATA); ArraySetAsSeries(C2, true);
  SetIndexBuffer(3, C3, INDICATOR_DATA); ArraySetAsSeries(C3, true);
  SetIndexBuffer(4, C4, INDICATOR_DATA); ArraySetAsSeries(C4, true);
  SetIndexBuffer(5, C5, INDICATOR_DATA); ArraySetAsSeries(C5, true);
  SetIndexBuffer(6, C6, INDICATOR_DATA); ArraySetAsSeries(C6, true);
  SetIndexBuffer(7, C7, INDICATOR_DATA); ArraySetAsSeries(C7, true);
  SetIndexBuffer(8, C8, INDICATOR_DATA); ArraySetAsSeries(C8, true);
  SetIndexBuffer(9, C9, INDICATOR_DATA); ArraySetAsSeries(C9, true);

  SetIndexBuffer(10, S0, INDICATOR_CALCULATIONS); ArraySetAsSeries(S0, true);
  SetIndexBuffer(11, S1, INDICATOR_CALCULATIONS); ArraySetAsSeries(S1, true);
  SetIndexBuffer(12, S2, INDICATOR_CALCULATIONS); ArraySetAsSeries(S2, true);
  SetIndexBuffer(13, S3, INDICATOR_CALCULATIONS); ArraySetAsSeries(S3, true);
  SetIndexBuffer(14, S4, INDICATOR_CALCULATIONS); ArraySetAsSeries(S4, true);
  SetIndexBuffer(15, S5, INDICATOR_CALCULATIONS); ArraySetAsSeries(S5, true);
  SetIndexBuffer(16, S6, INDICATOR_CALCULATIONS); ArraySetAsSeries(S6, true);
  SetIndexBuffer(17, S7, INDICATOR_CALCULATIONS); ArraySetAsSeries(S7, true);
  SetIndexBuffer(18, S8, INDICATOR_CALCULATIONS); ArraySetAsSeries(S8, true);
  SetIndexBuffer(19, S9, INDICATOR_CALCULATIONS); ArraySetAsSeries(S9, true);

  SetIndexBuffer(20, F0, INDICATOR_CALCULATIONS); ArraySetAsSeries(F0, true);
  SetIndexBuffer(21, F1, INDICATOR_CALCULATIONS); ArraySetAsSeries(F1, true);
  SetIndexBuffer(22, F2, INDICATOR_CALCULATIONS); ArraySetAsSeries(F2, true);
  SetIndexBuffer(23, F3, INDICATOR_CALCULATIONS); ArraySetAsSeries(F3, true);
  SetIndexBuffer(24, F4, INDICATOR_CALCULATIONS); ArraySetAsSeries(F4, true);
  SetIndexBuffer(25, F5, INDICATOR_CALCULATIONS); ArraySetAsSeries(F5, true);
  SetIndexBuffer(26, F6, INDICATOR_CALCULATIONS); ArraySetAsSeries(F6, true);
  SetIndexBuffer(27, F7, INDICATOR_CALCULATIONS); ArraySetAsSeries(F7, true);
  SetIndexBuffer(28, F8, INDICATOR_CALCULATIONS); ArraySetAsSeries(F8, true);
  SetIndexBuffer(29, F9, INDICATOR_CALCULATIONS); ArraySetAsSeries(F9, true);

  color colors[] =
  {
    clrRed, clrBlue, clrGreen, clrChocolate, clrMagenta, clrDodgerBlue, clrGoldenrod, clrIndigo, clrLightBlue, clrAliceBlue
  };

  int order = iSets.getSymbols(iSeries, InpSymbols);

  for (int k = 0; k < 10; k++)
  {
    if (k < order) 
    {
      PlotIndexSetInteger(k, PLOT_LINE_COLOR, 0, colors[k]);
      PlotIndexSetInteger(k, PLOT_DRAW_TYPE, DRAW_LINE);
      PlotIndexSetString(k, PLOT_LABEL, iSeries[k].name); 
      continue;
    }

    PlotIndexSetInteger(k, PLOT_DRAW_TYPE, DRAW_NONE);
  }

  IndicatorSetInteger(INDICATOR_DIGITS, 8);
  IndicatorSetString(INDICATOR_SHORTNAME, "MA " + InpSymbols);

  if (MQLInfoInteger(MQL_TESTER) == 0)
  {
    EventSetMillisecondTimer(1000);
  }
}

void OnTimer()
{
  if (History(iSeries) && iLastBars == ArraySize(C0)) 
  {
    EventKillTimer();
  }
Print(iLastBars, " = ", ArraySize(C0));  
  iLastBars = ArraySize(C0);
}

int OnCalculate(const int bars, const int counted, const int start, const double &price[])
{
  if (iStart == 0)
  {
    iStart = 1;
    ZeroMemory(C0); ZeroMemory(S0); ZeroMemory(F0); 
    ZeroMemory(C1); ZeroMemory(S1); ZeroMemory(F1); 
    ZeroMemory(C2); ZeroMemory(S2); ZeroMemory(F2); 
    ZeroMemory(C3); ZeroMemory(S3); ZeroMemory(F3); 
    ZeroMemory(C4); ZeroMemory(S4); ZeroMemory(F4); 
    ZeroMemory(C5); ZeroMemory(S5); ZeroMemory(F5); 
    ZeroMemory(C6); ZeroMemory(S6); ZeroMemory(F6); 
    ZeroMemory(C7); ZeroMemory(S7); ZeroMemory(F7); 
    ZeroMemory(C8); ZeroMemory(S8); ZeroMemory(F8); 
    ZeroMemory(C9); ZeroMemory(S9); ZeroMemory(F9); 
    History(iSeries);
    return bars;
  }

  if (bars != counted)
  {
    C0[0] = C0[1]; S0[0] = S0[1]; F0[0] = F0[1];
    C1[0] = C1[1]; S1[0] = S1[1]; F1[0] = F1[1]; 
    C2[0] = C2[1]; S2[0] = S2[1]; F2[0] = F2[1]; 
    C3[0] = C3[1]; S3[0] = S3[1]; F3[0] = F3[1]; 
    C4[0] = C4[1]; S4[0] = S4[1]; F4[0] = F4[1]; 
    C5[0] = C5[1]; S5[0] = S5[1]; F5[0] = F5[1]; 
    C6[0] = C6[1]; S6[0] = S6[1]; F6[0] = F6[1]; 
    C7[0] = C7[1]; S7[0] = S7[1]; F7[0] = F7[1]; 
    C8[0] = C8[1]; S8[0] = S8[1]; F8[0] = F8[1]; 
    C9[0] = C9[1]; S9[0] = S9[1]; F9[0] = F9[1]; 
  }

  if (iSets.getSyncQuotes(iSeries, 0, 2, PERIOD_CURRENT) > 0)
  {
    show(iSeries, S0, F0, C0, 0, 0);
    show(iSeries, S1, F1, C1, 1, 0);
    show(iSeries, S2, F2, C2, 2, 0);
    show(iSeries, S3, F3, C3, 3, 0);
    show(iSeries, S4, F4, C4, 4, 0);
    show(iSeries, S5, F5, C5, 5, 0);
    show(iSeries, S6, F6, C6, 6, 0);
    show(iSeries, S7, F7, C7, 7, 0);
    show(iSeries, S8, F8, C8, 8, 0);
    show(iSeries, S9, F9, C9, 9, 0);
  }

  return bars;
}

int History(SSets &series[])
{
  int count = iSets.getSyncQuotes(iSeries, 0, InpDepth, PERIOD_CURRENT);

  count = MathMin(count, iMaxBars);

  for (int n = count - 2; n >= 0; n--)
  {
    show(iSeries, S0, F0, C0, 0, n);
    show(iSeries, S1, F1, C1, 1, n);
    show(iSeries, S2, F2, C2, 2, n);
    show(iSeries, S3, F3, C3, 3, n);
    show(iSeries, S4, F4, C4, 4, n);
    show(iSeries, S5, F5, C5, 5, n);
    show(iSeries, S6, F6, C6, 6, n);
    show(iSeries, S7, F7, C7, 7, n);
    show(iSeries, S8, F8, C8, 8, n);
    show(iSeries, S9, F9, C9, 9, n);
  }

  ChartRedraw();
  
  return count;
}

void show(SSets &series[], double &S[], double &F[], double &C[], int index, int n)
{
  int count = ArraySize(series);
  
  if (index < count)
  {
    switch (InpMode)
    {
      case Change:
        S[n] = S[n + 1] + (series[index].rates[n].close - series[index].rates[n + 1].close) / series[index].rates[n].close;
        C[n] = iHelpers.EMA(S[n], C[n + 1], InpPeriod);
        break;
        
      case Average:
        S[n] = iHelpers.EMA(series[index].rates[n].close, v(S[n + 1], series[index].rates[n].close), InpPeriodSlow);
        F[n] = iHelpers.EMA(series[index].rates[n].close, v(F[n + 1], series[index].rates[n].close), InpPeriod);
        C[n] = F[n] - S[n];
        break;
    }
  }
}

double v(double v1, double v2)
{
  return v1 == 0 ? v2 : v1;
}